GAME_LUA = true
--[[
game state machine
	if state==0 then create pivots, set alphas to 0, load entities, state=1
	if state==1 then fade in pivots, rotate camera, run load text, reset self variables and state=2 at the end
	if state==2 then play, if pause then state=3, if die state=5
	if state==3 show pause menu, state=5
	if state==4 if resume state=2 and hide, if quit, exit. 
	if state==5 unload, state=0, return false
]]
game = {
states = { 	preload=0,
			start=1,
			load=2,
			play=3,
			pause=4,
			pause_wait=5,
			unload_mainmenu=6,
			death_overlay_fall_setup=7,
			death_overlay_fall=8,
			unload_gameover=9,
			win_setup=10,
			win_wait=11
		 },
consts = {	screen_scale  = {x=0.1661, y=0.1247, z=1},
			overlay_scale = {x=.00405, y=.00305, z=.003},
			overlay_alpha = 0.75,
			overlay_pos   = {0, 2, 0},
			win_anime_max = 240
},
explanation_frame_count=0,
explanation_text = {""},

do_state = function(dt)
	local gs = game.states
	if game.state == gs.preload then game.state = game.states.start
	elseif game.state == gs.start then game.start()
	elseif game.state == gs.load then level_setup(dt)
	elseif game.state == gs.play then game.play(dt)
	elseif game.state == gs.pause then game.pause()
	elseif game.state == gs.pause_wait then game.pause_wait()
	elseif game.state == gs.unload_mainmenu then
		game.kill_level()
		game.state = 0
		state = 2
	elseif game.state == gs.death_overlay_fall_setup then
		game.setup_death_overlay_fall()
	elseif game.state == gs.death_overlay_fall then
		game.death_overlay_fall()
	elseif game.state == gs.unload_gameover then
		E.delete_entity(death_overlay)
		E.delete_entity(death_overlay_border)
		game.kill_level()
		game.state = 0
		return false
	elseif game.state == gs.win_setup then game.win_setup()
	elseif game.state == gs.win_wait then game.win_wait()
	end
	return true
end,

load_pause_cursor = function()
	local rot = {rx=0, ry=0, rz=0}
	local pos = {x=0, y=0, z=0}
	game.pause_cursor = create_image_sprite(custom_cursor_file, rot, 0.0011, pos, true, custom_cursor)
	E.set_entity_flags(game.pause_cursor, E.entity_flag_hidden, true)
end,

update_pause_cursor_pos = function()
	game.ccur_xpos = (input.curs.px/800)*1.987 - 0.977
	game.ccur_ypos = ((600-input.curs.py)/600)*1.491 + 1.23
	player.entity.pause_cursor.pos = {game.ccur_xpos, game.ccur_ypos, player.entity.pause_cursor.pos[3]}
	_VP.refresh(player.entity)
end,

show_pause_cursor = function()
	E.set_entity_flags(game.pause_cursor, E.entity_flag_hidden, false)
end,

hide_pause_cursor = function()
	E.set_entity_flags(game.pause_cursor, E.entity_flag_hidden, true)
end,

explanation_setup = function()
	E.set_entity_position(light4, -5, 20, 20)
	game.explanation_text_size = 0
	game.explanation_strs = {}
	game.explanation_text_brush = E.create_brush()
	for i=1,#game.explanation_text do
		game.explanation_strs[i] = E.create_string("")
		E.set_string_fill(game.explanation_strs[i], game.explanation_text_brush)
		E.set_entity_position(game.explanation_strs[i], -35, 25-(i*5), 0)
		E.set_entity_scale(game.explanation_strs[i], 1.5, 1.5, 1.5)
		E.parent_entity(game.explanation_strs[i], pivot)
		game.explanation_text_size = game.explanation_text_size + #game.explanation_text[i]
	end
	E.set_brush_color(game.explanation_text_brush, 1.0, 1.0, 1.0, 1.0)
	game.explanation_line_num = 1
	game.explanation_letter_num = 1
	game.explanation_brush_alpha = 1.0
	game.state = game_state.explanation
end,

show_explanation = function()
	if game.explanation_frame_count < game.explanation_text_size+300 then
		if game.explanation_letter_num < #game.explanation_text[game.explanation_line_num] then
			E.set_string_text(game.explanation_strs[game.explanation_line_num],
							  string.sub(game.explanation_text[game.explanation_line_num], 0, game.explanation_letter_num))
			game.explanation_letter_num = game.explanation_letter_num + 1
		elseif game.explanation_line_num < #game.explanation_text then
			game.explanation_line_num = game.explanation_line_num + 1
			game.explanation_letter_num = 1
		end
		game.explanation_frame_count = game.explanation_frame_count + 1
	elseif game.explanation_frame_count < game.explanation_text_size+420 then
		E.set_brush_color(game.explanation_text_brush, 1.0, 1.0, 1.0, game.explanation_brush_alpha)
		game.explanation_brush_alpha = game.explanation_brush_alpha - 0.008
		game.explanation_frame_count = game.explanation_frame_count + 1
	else
		game.state = game_state.title_intro
	end
end,

kill_explanation = function()
	for i=1, #game.explanation_text do
		E.delete_entity(game.explanation_strs[i])
	end
end,

make_screen = function(screen_file, screen_scale, parent)
	local screen = create_image_sprite(screen_file, {rx=0, ry=0, rz=0}, 1, {x=0, y=0, z=0}, false, game_screens)
	E.set_entity_scale(screen, screen_scale.x, screen_scale.y, screen_scale.z)
	E.set_entity_flags(screen, E.entity_flag_hidden, true)
	if not parent then 	E.parent_entity(screen, pivot)
	else				E.parent_entity(screen, parent)
	end
	return screen
end,

start = function()
	E.set_entity_position(light,    0, 500,  1500)
	E.set_entity_position(light2,   0, 500, -1500)
	E.set_entity_position(light3, 500,   0,  1500)
	E.set_entity_position(light4, 500,   0, -1500)
	total_frames = 0
	level_setup()
		
	game.load_pause_cursor()

	pause_overlay_normal 	= game.make_screen(pause_overlay_file_normal, game.consts.overlay_scale, screen_overlay_pivot)
	pause_overlay_resume 	= game.make_screen(pause_overlay_file_resume, game.consts.overlay_scale, screen_overlay_pivot)
	pause_overlay_mainmenu 	= game.make_screen(pause_overlay_file_mainmenu, game.consts.overlay_scale, screen_overlay_pivot)
	pause_overlay_quit 		= game.make_screen(pause_overlay_file_quit, game.consts.overlay_scale, screen_overlay_pivot)
	E.set_entity_alpha(pause_overlay_normal, game.consts.overlay_alpha)
	E.set_entity_alpha(pause_overlay_resume, game.consts.overlay_alpha)
	E.set_entity_alpha(pause_overlay_mainmenu, game.consts.overlay_alpha)
	E.set_entity_alpha(pause_overlay_quit, game.consts.overlay_alpha)

	win_overlay_normal 		= game.make_screen(win_overlay_file_normal, game.consts.overlay_scale, screen_overlay_pivot)
	win_overlay_playagain	= game.make_screen(win_overlay_file_playagain, game.consts.overlay_scale, screen_overlay_pivot)
	win_overlay_mainmenu 	= game.make_screen(win_overlay_file_mainmenu, game.consts.overlay_scale, screen_overlay_pivot)
	win_overlay_quit 		= game.make_screen(win_overlay_file_quit, game.consts.overlay_scale, screen_overlay_pivot)
	
	if player then
		player.entity.pause_cursor 			=_VP.attach_entity(player.entity, game.pause_cursor, {game.consts.overlay_pos[1], game.consts.overlay_pos[2], 0.12})
		player.entity.pause_overlay_normal	=_VP.attach_entity(player.entity, pause_overlay_normal, {game.consts.overlay_pos[1], game.consts.overlay_pos[2], 0.1})
		player.entity.pause_overlay_resume	=_VP.attach_entity(player.entity, pause_overlay_resume, {game.consts.overlay_pos[1], game.consts.overlay_pos[2], 0.1})
		player.entity.pause_overlay_mainmenu=_VP.attach_entity(player.entity, pause_overlay_mainmenu, {game.consts.overlay_pos[1], game.consts.overlay_pos[2], 0.1})
		player.entity.pause_overlay_quit	=_VP.attach_entity(player.entity, pause_overlay_quit, {game.consts.overlay_pos[1], game.consts.overlay_pos[2], 0.1})
		
		player.entity.win_overlay_normal	=_VP.attach_entity(player.entity, win_overlay_normal, {game.consts.overlay_pos[1], game.consts.overlay_pos[2], 0.1})
		player.entity.win_overlay_playagain	=_VP.attach_entity(player.entity, win_overlay_playagain, {game.consts.overlay_pos[1], game.consts.overlay_pos[2], 0.1})
		player.entity.win_overlay_mainmenu	=_VP.attach_entity(player.entity, win_overlay_mainmenu, {game.consts.overlay_pos[1], game.consts.overlay_pos[2], 0.1})
		player.entity.win_overlay_quit		=_VP.attach_entity(player.entity, win_overlay_quit, {game.consts.overlay_pos[1], game.consts.overlay_pos[2], 0.1})
	end

	game.state = game.states.load
end,

play = function(dt)
	player.do_state(dt)
	_enemy.do_states(dt)
	_boss.do_state(dt)
	_sslash.do_states(dt)
	E.turn_entity(final_obj, 0, 1, 0)
	if input.key.p then
		input.key.p = false
		game.state = game.states.pause
	end
	--if not totalt then totalt=total_dt+3
	--elseif totalt<total_dt then totalt=totalt+3 end
	--E.set_entity_alpha(wallgroup, (3-totalt+total_dt)/3)
end,

pause = function()
	player.display_all_stats()
	game.show_pause_cursor()
	game.make_pause_overlay_visible(pause_overlay_normal)
	game.state = game.states.pause_wait
end,

pause_wait = function()
	game.update_pause_cursor_pos()
	local ic = input.curs
	local prbb = pause_resume_btn_bounds
	local pmbb = pause_mainmenu_btn_bounds
	local pqbb = pause_quit_btn_bounds
	if ic.px > prbb.x[1] and ic.px < prbb.x[2] and ic.py > prbb.y[1] and ic.py < prbb.y[2] then
		game.make_pause_overlay_visible(pause_overlay_resume)
		if ic.click then 
			player.hide_all_stats()
			game.hide_pause_cursor()
			game.make_pause_overlay_visible()
			game.state = game.states.play 
		end
	elseif ic.px > pmbb.x[1] and ic.px < pmbb.x[2] and ic.py > pmbb.y[1] and ic.py < pmbb.y[2] then
		game.make_pause_overlay_visible(pause_overlay_mainmenu)
		if ic.click then
			game.hide_pause_cursor()
			game.make_pause_overlay_visible()
			game.state = game.states.unload_mainmenu 
		end
	elseif ic.px > pqbb.x[1] and ic.px < pqbb.x[2] and ic.py > pqbb.y[1] and ic.py < pqbb.y[2] then
		game.make_pause_overlay_visible(pause_overlay_quit)
		if ic.click then
			os.execute("TASKKILL /F /IM GlovePIE.exe || cls")
			E.exit()
		end
	elseif input.key.p then
		player.hide_all_stats()
		game.hide_pause_cursor()
		game.make_pause_overlay_visible()
		game.state = game.states.play 
		input.key.p = false
	else
		game.make_pause_overlay_visible(pause_overlay_normal)
		input.curs.click=false
	end
end,

make_pause_overlay_visible = function(pause_overlay)
	E.set_entity_flags(pause_overlay_normal, E.entity_flag_hidden, true)
	E.set_entity_flags(pause_overlay_resume, E.entity_flag_hidden, true)
	E.set_entity_flags(pause_overlay_mainmenu, E.entity_flag_hidden, true)
	E.set_entity_flags(pause_overlay_quit, E.entity_flag_hidden, true)
	if pause_overlay then
		E.set_entity_flags(pause_overlay, E.entity_flag_hidden, false)
	end
end,

setup_death_overlay_fall = function()
	death_overlay_scale = 0.0786
	local death_scale 	= {x=death_overlay_scale, y=death_overlay_scale, z=death_overlay_scale}
	death_overlay		= game.make_screen(death_overlay_file, death_scale, screen_overlay_pivot)
	death_overlay_border = game.make_screen(death_overlay_border_file, death_scale, screen_overlay_pivot)
	E.set_entity_flags(death_overlay, E.entity_flag_hidden, false)
	E.set_entity_flags(death_overlay_border, E.entity_flag_hidden, false)
	--E.set_entity_alpha(death_overlay, game.consts.overlay_alpha+0.10)
	player.entity.death_overlay			=_VP.attach_entity(player.entity, death_overlay, {0, 2, 0})
	player.entity.death_overlay_border	=_VP.attach_entity(player.entity, death_overlay_border, {0, 2, 0})
	
	game.state = game.states.death_overlay_fall
end,

death_overlay_fall = function()
	if death_overlay_scale > 0.0018 then
		death_overlay_scale = death_overlay_scale - 0.0004
		E.set_entity_scale(death_overlay, death_overlay_scale, death_overlay_scale, death_overlay_scale)
		E.set_entity_scale(death_overlay_border, death_overlay_scale, death_overlay_scale, death_overlay_scale)			
	elseif death_overlay_scale > 0.00001 then
		death_overlay_scale = death_overlay_scale - 0.0004
	else
		game.state = game.states.unload_gameover
	end
end,

kill_level = function()
	-- clear player stats before deleting player
	player.reset_stats()
	
	-- delete pivots
	E.delete_entity(walldecorgroup)
	E.delete_entity(sslashburnmarkgroup)
	E.delete_entity(sslashexpoflashgroup)
	E.delete_entity(playergroup)
	E.delete_entity(stringgroup)
	E.delete_entity(bulletgroup)
	E.delete_entity(enemyfire)
	E.delete_entity(enemydeadgroup)
	E.delete_entity(enemygroup)
	E.delete_entity(wallgroup)
	E.delete_entity(itemgroup)
	E.delete_entity(ckptgroup)
	E.delete_entity(avatargroup)
	
	-- reset input values
	input.x=0
	input.y=0
	input.turn=0
	input.key = {q=false, w=false, e=false, a=false, s=false, d=false, z=false, left=false, right=false, space=false, u=false,
				 i=false, o=false, p=false, j=false, k=false, l=false, n=false, m=false, comma=false}
	input.key.num={false, false, false, false, false, false, false, false, false, false}
			
	-- reset all tables
	walls 				= {}
	firstaidkits 		= {}
	saberenergykits 	= {}
	_enemy.enemies 		= {}
	-- if _enemy.shots[1] then 
	-- for i,v in pairs(_enemy.shots) do
		-- _enemy.del_shot(i, v)
	-- end
	-- end
	-- if _sslash.sslashes[1] then
	-- for j,u in pairs(_sslash.sslashes) do
		-- _sslash.del(j, u)
	-- end
	-- end
	_enemy.shots 		= {}
	_enemy.sight_shots 	= {}
	_sslash.sslashes 	= {}
end,

win_setup = function()
	if game.win_anime_count < game.consts.win_anime_max then
		E.set_entity_alpha(pivot, (game.consts.win_anime_max-game.win_anime_count)/game.consts.win_anime_max)
		game.win_anime_count = game.win_anime_count + 1
	else
		game.kill_level()
		E.set_entity_alpha(pivot, 100)
		game.make_win_overlay_visible(win_overlay_normal)
		game.show_pause_cursor()
		game.state = game.states.win_wait
	end
end,

win_wait = function()
	local ic = input.curs
	local wpbb = win_playagain_btn_bounds
	local wmbb = win_mainmenu_btn_bounds
	local wqbb = win_quit_btn_bounds
	if ic.px > wpbb.x[1] and ic.px < wpbb.x[2] and ic.py > wpbb.y[1] and ic.py < wpbb.y[2] then
		game.make_win_overlay_visible(win_overlay_playagain)
		if ic.click then
			game.kill_level()
			game.make_win_overlay_visible()
			game.state = game.states.start 
		end
	elseif ic.px > wmbb.x[1] and ic.px < wmbb.x[2] and ic.py > wmbb.y[1] and ic.py < wmbb.y[2] then
		game.make_win_overlay_visible(win_overlay_mainmenu)
		if ic.click then
			game.make_win_overlay_visible()
			game.state = game.states.unload_mainmenu 
		end
	elseif ic.px > wqbb.x[1] and ic.px < wqbb.x[2] and ic.py > wqbb.y[1] and ic.py < wqbb.y[2] then
		game.make_win_overlay_visible(win_overlay_quit)
		if ic.click then
			os.execute("TASKKILL /F /IM GlovePIE.exe || cls")
			E.exit()
		end
	else
		game.make_win_overlay_visible(win_overlay_normal)
		input.curs.click=false
	end
	game.update_pause_cursor_pos()
end,

make_win_overlay_visible = function(win_overlay)
	E.set_entity_flags(win_overlay_normal, E.entity_flag_hidden, true)
	E.set_entity_flags(win_overlay_playagain, E.entity_flag_hidden, true)
	E.set_entity_flags(win_overlay_mainmenu, E.entity_flag_hidden, true)
	E.set_entity_flags(win_overlay_quit, E.entity_flag_hidden, true)
	if win_overlay then
		E.set_entity_flags(win_overlay, E.entity_flag_hidden, false)
	end
end,

state=0
}
